@@ -1,3 +1,7 @@
+"""给你的 Python 迭代器加上魔法
+
+一个 Python 迭代器的包装器，使其具有与Rust中的其他方法类似的风格，以提高迭代器操作的一致性和代码的可读性。
+"""
 import operator
 import sys
 from collections.abc import (
@@ -24,6 +28,7 @@ from itertools import (
 )
 from typing import (
     Any,
+    Generic,
     overload,
     Optional,
     Union,
@@ -35,19 +40,28 @@ from typing import (
 
 __all__ = ["ArkoWrapper"]
 
+M = TypeVar("M")
 T = TypeVar("T")
+E = TypeVar("E")
+C = TypeVar("C")
 
 
-class ArkoWrapper(object):
-    __root__: Iterable[Any]
+class ArkoWrapper(Generic[T]):
+    """一个 Python 迭代器的包装器
+
+    Attributes:
+        iterable: 需要被 Wrap 的 Object。 可以是一个迭代器或者其它任何Object。
+        max_operate_times: Wrapper 操作次数的上限。用于限制无限的迭代器。
+    """
+    __root__: Iterable[T]
     _max: int
 
     __slots__ = '__root__', '_max'
 
     # noinspection PyTypeChecker
     def __init__(
-            self, iterable: Optional[Any] = None, *,
-            max_num: Optional[int] = 2 ** 20
+            self, iterable: Optional[Union[Iterable[T], T]] = None, *,
+            max_operate_times: Optional[int] = sys.maxsize
     ) -> NoReturn:
         if isinstance(iterable, Iterable):
             self.__root__ = iterable
@@ -56,29 +70,40 @@ class ArkoWrapper(object):
         else:
             self.__root__ = [iterable]
 
-        if max_num <= 0:
-            raise ValueError(f"Requires a positive number: {max_num}")
-        self._max = max_num
+        if max_operate_times <= 0:
+            raise ValueError(f"Requires a positive number: {max_operate_times}")
+        self._max = max_operate_times
 
     def __str__(self) -> str:
         return str(self.__root__)
 
     def __repr__(self) -> str:
-        return f"<ArkoWrapper:{self.__root__}>"
+        return f"<{self.__class__.__name__}:{self.__root__}>"
 
-    def _tee(self) -> Iterable:
+    def _tee(self) -> Iterable[T]:
+        """将已有迭代器分裂一次"""
         result, self.__root__ = tee(self.__root__)
         return result
 
-    def _max_gen(self) -> Generator:
+    def _max_gen(self) -> Generator[T]:
+        """将自己的迭代器现在某个范围内"""
         iter_values = iter(self._tee())
-        for _ in range(self._max):
-            try:
+        try:
+            for _ in range(self._max):
                 yield next(iter_values)
-            except StopIteration:
-                break
+        except StopIteration:
+            ...
+
+    def __add__(self: M, other: E) -> "M[Union[T, E]]":
+        """实现加法操作。返回两个实列组成的新的迭代器的ArkoWrapper
+
+        Args:
+            other(Any): 相加的实列。
+
+        Returns:
+            ArkoWrapper: 返回两个实列组成的新的迭代器的ArkoWrapper
+        """
 
-    def __add__(self, other: Any) -> "ArkoWrapper":
         def generate() -> Generator:
             yield from self._tee()
             if isinstance(other, ArkoWrapper):
@@ -88,10 +113,10 @@ class ArkoWrapper(object):
             else:
                 yield other
 
-        return ArkoWrapper(generate())
+        return self.__class__(generate())
 
-    def __radd__(self, other: Any) -> "ArkoWrapper":
-        """radd魔法方法"""
+    def __radd__(self: M, other: E) -> "M[Union[T, E]]":
+        """实现反射加法操作。"""
 
         def generate() -> Generator:
             if isinstance(other, ArkoWrapper):
@@ -102,9 +127,10 @@ class ArkoWrapper(object):
                 yield other
             yield from self._tee()
 
-        return ArkoWrapper(generate())
+        return self.__class__(generate())
 
     def __eq__(self, other: Any) -> bool:
+        """定义操作符(==)的行为。"""
         if any([
             isinstance(other, ArkoWrapper) and self.__root__ == other.__root__,
             isinstance(other, Iterable) and self.__root__ == other
@@ -117,22 +143,22 @@ class ArkoWrapper(object):
             return True
         return self == [other]
 
-    def __copy__(self) -> "ArkoWrapper":
-        return ArkoWrapper(self._tee())
+    def __copy__(self: M) -> "M":
+        """定义对类的实例使用 copy.copy() 时的行为"""
+        return self.__class__(self._tee())
 
-    def __getitem__(self, index: Any) -> Any:
+    def __getitem__(self, index: Any) -> T:
+        """定义对容器中某一项使用 self[key] 的方式进行读取操作时的行为"""
         if isinstance(index, slice):
             if '__getitem__' in dir(self.__root__):
                 # noinspection PyUnresolvedReferences
                 return self.__root__[index]
-            if (
-                    all([index.start, index.step, index.stop]) and
-                    index.start * index.step * index.stop > 0
-            ):
+            try:
                 return self.slice(
                     *filter(None, [index.start, index.step, index.stop])
                 )
-            return list(self._max_gen()).__getitem__(index)
+            except ValueError:
+                return list(self._max_gen()).__getitem__(index)
         try:
             if (index := int(index)) > 0:
                 target = self._tee()
@@ -151,12 +177,15 @@ class ArkoWrapper(object):
             raise IndexError("Unsupported indexing for iterable")
 
     def __index__(self) -> int:
+        """实现当对象用于切片表达式时到一个整数的类型转换。"""
         return self.__len__()
 
-    def __iter__(self) -> Any:
+    def __iter__(self) -> T:
+        """返回当前的迭代器。"""
         return self._tee().__iter__()
 
     def __len__(self) -> int:
+        """返回当前的迭代器的长度，如果无限的话，则回返回最大操作次数。"""
         if isinstance(self.__root__, Sized):
             return len(list(self._tee()))
         else:
@@ -167,43 +196,49 @@ class ArkoWrapper(object):
                     return self._max
             return length
 
-    def __matmul__(self, other: Any) -> Any:
+    def __matmul__(self, other: Any) -> T:
+        """定义操作符(@)的行为。"""
         return self.__getitem__(other)
 
-    def __mul__(self, times: Union[int, float, str]) -> "ArkoWrapper":
+    def __mul__(self: M, times: Union[int, float, str]) -> "M":
+        """实现乘法操作"""
         try:
             if (times := int(float(times))) <= 0:
                 raise ValueError(f"'times' cannot be negative: {times}")
-            return ArkoWrapper(
+            return self.__class__(
                 chain.from_iterable(repeat(tuple(self._tee()), times))
             )
         except (ValueError, TypeError):
             raise TypeError(f"Unsupported Type: {type(times)}.")
 
-    def __neg__(self) -> "ArkoWrapper":
+    def __neg__(self: M) -> "M":
+        """定义取负操作"""
         return self.__reversed__()
 
-    def __reversed__(self) -> "ArkoWrapper":
+    def __reversed__(self: M) -> "M":
+        """定义反转"""
         if isinstance(self.__root__, Reversible):
             from copy import deepcopy as copy
-            return ArkoWrapper(reversed(copy(self.__root__)))
+            return self.__class__(reversed(copy(self.__root__)))
         else:
             try:
-                return ArkoWrapper(reversed(list(self._max_gen())))
+                return self.__class__(reversed(list(self._max_gen())))
             except (ValueError, TypeError):
                 raise TypeError(
                     f"The iter '{self.__root__}' is not 'Reversible.'")
 
-    def __rshift__(self, target: Any) -> Any:
+    def __rshift__(self: M, target: Any) -> C:
+        """实现右移位运算符 >>"""
         if isinstance(target, type) or callable(target):
-            return self.collect(target)
+            result: C = self.collect(target)
         elif isinstance(target, Sequence):
-            return self.collect(type(target))
+            result: C = self.collect(type(target))
         else:
             raise ValueError(f"Unsupported value or type: '{target}'")
+        return result
 
     @property
-    def root(self) -> Iterable[Any]:
+    def root(self) -> Iterable[T]:
         return self.__root__
 
     @property
@@ -211,22 +246,24 @@ class ArkoWrapper(object):
         return self.__len__()
 
     @property
-    def max(self) -> int:
+    def max_operate_time(self) -> int:
+        """最大操作次数，防止无线递归"""
         return self._max
 
-    @max.setter
-    def max(self, max_num: int):
-        if max_num <= 0:
-            raise ValueError(f"Requires a positive number: {max_num}")
-        self._max = max_num
+    @max_operate_time.setter
+    def max_operate_time(self, max_operate_time: int):
+        if max_operate_time <= 0:
+            raise ValueError(f"Requires a positive number: {max_operate_time}")
+        self._max = max_operate_time
 
     def accumulate(
-            self, func: Optional[Callable[[...], Any]] = operator.add, *,
+            self: M, func: Optional[Callable[[T, T], C]] = operator.add, *,
             initial: Optional[int] = None
-    ) -> "ArkoWrapper":
-        return ArkoWrapper(accumulate(self._tee(), func, initial=initial))
+    ) -> "M[C]":
+        return self.__class__(accumulate(self._tee(), func, initial=initial))
 
     def all(self) -> bool:
+        """如果所有元素均为真值（或root为空）则返回 True"""
         iter_values = iter(self._tee())
         try:
             while bool(next(iter_values)):
@@ -236,6 +273,7 @@ class ArkoWrapper(object):
             return True
 
     def any(self) -> bool:
+        """如果任一元素为真值则返回 True。 若root为空，返回 False"""
         iter_values = iter(self._tee())
         try:
             while not bool(next(iter_values)):
@@ -244,36 +282,38 @@ class ArkoWrapper(object):
         except StopIteration:
             return False
 
-    def chain(self, *iterables: Iterable) -> "ArkoWrapper":
-        return ArkoWrapper(chain(self._tee(), *iterables))
+    def chain(self: M, *iterables: Iterable[E]) -> "M[Union[T, E]]":
+        """创建一个迭代器，它首先返回第一个可迭代对象中所有元素，接着返回下一个可迭代对象中所有元素，直到耗尽所有可迭代对象中的元素。"""
+        return self.__class__(chain(self._tee(), *iterables))
 
     def collect(
             self,
-            func: Optional[Callable[[Iterable, ...], T]] = list,
+            func: Optional[Callable[[Iterable[T], ...], C]] = list,
             *args, **kwargs
-    ) -> T:
+    ) -> C:
+        """以整个迭代器作为参数，于给定的 func 函数中进行运算"""
         return func(self._tee(), *args, **kwargs)
 
-    def combinations(self, r: int = 2) -> "ArkoWrapper":
-        return ArkoWrapper(combinations(self._tee(), r))
+    def combinations(self: M, r: int = 2) -> "M[combinations[tuple[T, T]]]":
+        return self.__class__(combinations(self._tee(), r))
 
-    def combinations_with_replacement(self, r: int = 2) -> "ArkoWrapper":
-        return ArkoWrapper(combinations_with_replacement(self._tee(), r))
+    def combinations_with_replacement(self: M, r: int = 2) -> "M":
+        return self.__class__(combinations_with_replacement(self._tee(), r))
 
-    def compress(self, selectors: Iterable) -> "ArkoWrapper":
-        return ArkoWrapper(compress(self._tee(), selectors))
+    def compress(self: M, selectors: Iterable) -> "M":
+        return self.__class__(compress(self._tee(), selectors))
 
-    def cycle(self) -> "ArkoWrapper":
-        return ArkoWrapper(cycle(self._tee()))
+    def cycle(self: M) -> "M":
+        return self.__class__(cycle(self._tee()))
 
-    def drop_while(self, func: Callable) -> "ArkoWrapper":
-        return ArkoWrapper(dropwhile(func, self._tee()))
+    def drop_while(self: M, func: Callable) -> "M":
+        return self.__class__(dropwhile(func, self._tee()))
 
-    def enumerate(self) -> "ArkoWrapper":
-        def generator() -> Generator[tuple[int, Any]]:
+    def enumerate(self: M) -> "M[tuple[int, T]]":
+        def generator() -> Generator[tuple[int, T]]:
             iter_values = iter(self._tee())
             index = 0
-            while True:
+            for _ in range(self.max_operate_time):
                 try:
                     value = next(iter_values)
                     yield index, value
@@ -281,13 +321,13 @@ class ArkoWrapper(object):
                 except StopIteration:
                     break
 
-        return ArkoWrapper(generator())
+        return self.__class__(generator())
 
-    def filter(self, func: Callable[[...], Any]) -> "ArkoWrapper":
-        return ArkoWrapper(filter(func, self._tee()))
+    def filter(self: M, func: Callable[[T], Any]) -> "M[T]":
+        return self.__class__(filter(func, self._tee()))
 
-    def filter_false(self, func: Callable[[...], Any]) -> "ArkoWrapper":
-        return ArkoWrapper(filterfalse(func, self._tee()))
+    def filter_false(self: M, func: Callable[[T], Any]) -> "M[T]":
+        return self.__class__(filterfalse(func, self._tee()))
 
     def find(
             self, target: Any, *, full: Optional[bool] = False
@@ -298,7 +338,7 @@ class ArkoWrapper(object):
                 if not full:
                     break
 
-    def flat(self, depth: int = -1) -> "ArkoWrapper":
+    def flat(self: M, depth: int = -1) -> "M":
         def generate(iterator: Iterable, times: int) -> Generator:
             for i in iterator:
                 if isinstance(i, Iterable) and not isinstance(i, str) and times:
@@ -306,19 +346,31 @@ class ArkoWrapper(object):
                 else:
                     yield i
 
-        return ArkoWrapper(generate(self._tee(), depth))
+        return self.__class__(generate(self._tee(), depth))
 
-    def group(self, n: int, fill_value: Any = None) -> "ArkoWrapper":
+    def group(self: M, n: int, fill_value: Any = None) -> "M":
         iter_values = [iter(self._tee())] * n
-        return ArkoWrapper(zip_longest(*iter_values, fillvalue=fill_value))
+        return self.__class__(zip_longest(*iter_values, fillvalue=fill_value))
 
-    def map(self, func: Callable) -> "ArkoWrapper":
-        return ArkoWrapper(map(func, self._tee()))
+    def join(self, sep: str = ', ') -> str:
+        iter_values = iter(self._tee())
+        result = ''
+        try:
+            for _ in range(self.max_operate_time):
+                value = next(iter_values)
+                result += f"{sep}{value}"
+        except StopIteration:
+            ...
+        return result
+
+    def map(self: M, func: Callable) -> "M":
+        return self.__class__(map(func, self._tee()))
 
     def mutate(
-            self, func: Callable[..., Iterable] = list, *args, **kwargs
-    ) -> "ArkoWrapper":
-        return ArkoWrapper(func(self._tee(), *args, **kwargs))
+            self: M, func: Callable[[Iterable[T], ...], Iterable] = list,
+            *args, **kwargs
+    ) -> "M":
+        return self.__class__(func(self._tee(), *args, **kwargs))
 
     def print(
             self,
@@ -344,7 +396,7 @@ class ArkoWrapper(object):
             print_func(len(end) * '\b')
         return self
 
-    def range(self, start: Optional[int] = 0) -> "ArkoWrapper":
+    def range(self: M, start: Optional[int] = 0) -> "M":
         def generator() -> Generator[int]:
             iter_values = iter(self._tee())
             index = start
@@ -353,22 +405,22 @@ class ArkoWrapper(object):
                     next(iter_values)
                     index -= 1
                 index = start
-                while True:
+                for _ in range(self.max_operate_time):
                     next(iter_values)
                     yield index
                     index += 1
             except StopIteration:
                 return
 
-        return ArkoWrapper(generator())
+        return self.__class__(generator())
 
     def remove(self, target: Any) -> "ArkoWrapper":
         def generator():
             iter_values = iter(self._tee())
             is_sequence = isinstance(target, Sequence)
             removed = False
-            while True:
-                try:
+            try:
+                for _ in range(self.max_operate_time):
                     value = next(iter_values)
                     if (
                             (is_sequence and value in target)
@@ -378,15 +430,16 @@ class ArkoWrapper(object):
                         continue
                     else:
                         yield value
-                except StopIteration:
-                    ...
+            except StopIteration:
+                ...
+
         return ArkoWrapper(generator())
 
     def repeat(
-            self, times: Optional[Union[int, float, str]] = None
-    ) -> "ArkoWrapper":
+            self: M, times: Optional[Union[int, float, str]] = None
+    ) -> "M[T]":
         if times is None:
-            return ArkoWrapper(
+            return self.__class__(
                 chain.from_iterable(
                     repeat(tuple(self._tee()), self._max + 1)
                 )
@@ -397,27 +450,27 @@ class ArkoWrapper(object):
         ]):
             if (time := int(times)) <= 0:
                 raise ValueError(f"'times' cannot be negative: {times}")
-            return ArkoWrapper(
+            return self.__class__(
                 chain.from_iterable(repeat(tuple(self._tee()), time + 1))
             )
         else:
             raise TypeError(f"Unsupported Type: {type(times)}.")
 
-    def reverse(self) -> "ArkoWrapper":
+    def reverse(self: M) -> "M":
         return self.__reversed__()
 
     @overload
-    def slice(self, stop: int) -> "ArkoWrapper":
+    def slice(self: M, stop: int) -> "M":
         ...
 
     @overload
     def slice(
-            self, start: int, stop: int, step: Optional[int] = 1
-    ) -> "ArkoWrapper":
+            self: M, start: int, stop: int, step: Optional[int] = 1
+    ) -> "M":
         ...
 
-    def slice(self, *args) -> "ArkoWrapper":
-        return ArkoWrapper(islice(self._tee(), *args))
+    def slice(self: M, *args) -> "M":
+        return self.__class__(islice(self._tee(), *args))
 
     def search(self, sub: Sized) -> Generator[int]:
         target = self.tee()
@@ -437,35 +490,37 @@ class ArkoWrapper(object):
             if target[i] == sub[j]:
                 j += 1
             if j == len(sub):
-                yield i - (j - 1)
+                yield i - j + 1
                 j = partial[j - 1]
 
     def sort(
-            self, key: Optional[Callable] = None, reverse: bool = False
-    ) -> "ArkoWrapper":
-        return ArkoWrapper(sorted(self._tee(), key=key, reverse=reverse))
+            self: M, key: Optional[Callable] = None, reverse: bool = False
+    ) -> "M":
+        return self.__class__(sorted(self._tee(), key=key, reverse=reverse))
 
-    def starmap(self, func: Callable) -> "ArkoWrapper":
-        return ArkoWrapper(starmap(func, self._tee()))
+    def starmap(self: M, func: Callable) -> "M":
+        return self.__class__(starmap(func, self._tee()))
 
     def tee(
-            self, n: Optional[int] = None
-    ) -> Union["ArkoWrapper", Generator["ArkoWrapper"]]:
+            self: M, n: Optional[int] = None
+    ) -> Union["M", Generator["M"]]:
         if n is None:
-            return ArkoWrapper(self._tee())
-        return (ArkoWrapper(item) for item in tee(self.__root__, n))
+            return self.__class__(self._tee())
+        return (self.__class__(item) for item in tee(self.__root__, n))
 
-    def unique(self) -> "ArkoWrapper":
+    def unique(self: M) -> "M":
         def generator() -> Generator:
             for k, g in groupby(self._tee()):
                 yield k
 
-        return ArkoWrapper(generator())
+        return self.__class__(generator())
 
-    def unwrap(self) -> Iterable:
+    def unwrap(self) -> Iterable[T]:
         return self._tee()
 
-    def zip(self, *iterables, strict: Optional[bool] = False) -> "ArkoWrapper":
+    def zip(
+            self: M, *iterables: Iterable[E], strict: Optional[bool] = False
+    ) -> "M[Union[T, E]]":
         if sys.version_info >= (3, 10):
-            return ArkoWrapper(zip(self._tee(), *iterables, strict=strict))
-        return ArkoWrapper(zip(self._tee(), *iterables))
+            return self.__class__(zip(self._tee(), *iterables, strict=strict))
+        return self.__class__(zip(self._tee(), *iterables))
